﻿using MVR.FileManagementSecure;
using SimpleJSON;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using static FreeControllerV3;

namespace Rak.VAM.ControllerPresets
{
    public class ControllerPresets : MVRScript
    {
        private List<ControllerPreset> ControllerPresetsList;
        private JSONStorableStringChooser ActivatePreset;
        private MainScreen MainScreen;
        public static string SAVEDIR = SuperController.singleton.savesDirResolved + "ControllerPresets";
        public bool AtomIsMale { get; private set; }
        public bool ListenForShortcutKeys { get; set; }
        public override void Init()
        {
            var geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            var morphControl = character.morphsControlUI;
            var morphList = morphControl.GetMorphUids();
            int female = 0;
            int male = 0;
            for (int count = 0; count < morphList.Count; count++)
            {
                if (morphList[count].ToLower().Contains("female"))
                    female++;
                else if (morphList[count].ToLower().Contains("male"))
                    male++;
            }
            if (male > female)
                AtomIsMale = true;
            else
                AtomIsMale = false;
            ControllerPresetsList = new List<ControllerPreset>();
            MainScreen = new MainScreen(this);
            ListenForShortcutKeys = true;
            ActivatePreset = new JSONStorableStringChooser("Activate Preset", GetPresetNameList(), "", "Activate Preset", ActivatePresetChanged);
            RegisterStringChooser(ActivatePreset);
            MainScreen.RefreshScreen();
        }
        private void ActivatePresetChanged(string newValue)
        {
            for (int count = 0; count < ControllerPresetsList.Count; count++)
            {
                if (ControllerPresetsList[count].Name.Equals(newValue))
                {
                    ControllerPresetsList[count].Activate();
                    return;
                }
            }
        }
        private bool _ControlDown;
        private void FixedUpdate()
        {
            if (ListenForShortcutKeys)
            {
                if (Input.GetKeyDown(KeyCode.LeftControl))
                {
                    _ControlDown = true;
                }
                else if (Input.GetKeyUp(KeyCode.LeftControl))
                    _ControlDown = false;
                if (Input.GetKeyDown(KeyCode.Alpha1))
                    ShortcutKeyPushed(1);
                else if (Input.GetKeyDown(KeyCode.Alpha2))
                    ShortcutKeyPushed(2);
                else if (Input.GetKeyDown(KeyCode.Alpha3))
                    ShortcutKeyPushed(3);
                else if (Input.GetKeyDown(KeyCode.Alpha4))
                    ShortcutKeyPushed(4);
                else if (Input.GetKeyDown(KeyCode.Alpha5))
                    ShortcutKeyPushed(5);
                else if (Input.GetKeyDown(KeyCode.Alpha6))
                    ShortcutKeyPushed(6);
                else if (Input.GetKeyDown(KeyCode.Alpha7))
                    ShortcutKeyPushed(7);
                else if (Input.GetKeyDown(KeyCode.Alpha8))
                    ShortcutKeyPushed(8);
                else if (Input.GetKeyDown(KeyCode.Alpha9))
                    ShortcutKeyPushed(9);
            }
            MainScreen.ManualUpdate();
        }
        private void ShortcutKeyPushed(int key)
        {
            if (!_ControlDown)
                return;
            for (int count = 0; count < ControllerPresetsList.Count; count++)
            {
                if (ControllerPresetsList[count].ShortcutKey == key)
                {
                    ControllerPresetsList[count].Activate();
                    return;
                }
            }
        }
        public ControllerPreset GetPreset(int index)
        {
            return ControllerPresetsList[index];
        }
        public List<string> GetPresetNameList()
        {
            List<string> list = new List<string>();
            for (int count = 0; count < ControllerPresetsList.Count; count++)
                list.Add(ControllerPresetsList[count].Name);
            return list;
        }
        public override JSONClass GetJSON(bool includePhysical = false, bool includeAppearance = false, bool forceStore = true)
        {
            JSONClass jsc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            needsStore = true;
            JSONArray presetArray = new JSONArray();
            for (int count = 0; count < ControllerPresetsList.Count; count++)
            {
                presetArray.Add(ControllerPresetsList[count].GetJSON());
            }
            jsc["ControllerPresetsList"] = presetArray;
            return jsc;
        }
        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = false, bool restoreAppearance = false, JSONArray presetAtoms = null, bool setMissingToDefault = true)
        {
            JSONArray presetsArrayJSON = jc["ControllerPresetsList"].AsArray;
            ControllerPresetsList = new List<ControllerPreset>();
            for (int count = 0; count < presetsArrayJSON.Count; count++)
            {
                JSONClass presetJSON = (JSONClass)presetsArrayJSON[count];
                ControllerPreset importedPreset = ControllerPreset.GetDefault(this);
                importedPreset.Name = presetJSON["Name"];
                importedPreset.ShortcutKey = presetJSON["ShortcutKey"].AsInt;
                var keys = importedPreset.ControllerStates.Keys.ToList();
                JSONClass statesJSON = presetJSON["ControllerStates"].AsObject;
                for (int conCount = 0; conCount < keys.Count; conCount++)
                {
                    string key = keys[conCount];
                    var state = importedPreset.ControllerStates[key];
                    state.PositionState = statesJSON[key]["PositionState"].AsInt;
                    state.RotationState = statesJSON[key]["RotationState"].AsInt;
                }
                JSONClass interactablesJSON = presetJSON["ControllersInteractable"].AsObject;
                keys = importedPreset.ControllersInteractable.Keys.ToList();
                for (int conCount = 0; conCount < keys.Count; conCount++)
                {
                    string key = keys[conCount];
                    importedPreset.ControllersInteractable[key] = interactablesJSON[key].AsBool;
                }
                
                JSONClass armedRecordJSON = presetJSON["ControllersArmedForRecord"].AsObject;
                keys = importedPreset.ControllersArmedForRecord.Keys.ToList();
                for (int conCount = 0; conCount < keys.Count; conCount++)
                {
                    string key = keys[conCount];
                    importedPreset.ControllersArmedForRecord[key] = armedRecordJSON[key].AsBool;
                }
                ControllerPresetsList.Add(importedPreset);
            }
            UpdateChoices();
            MainScreen.RefreshScreen();
        }
        public void UpdateChoices()
        {
            ActivatePreset.choices = GetPresetNameList();
        }
        public int CreateNewPreset()
        {
            ControllerPresetsList.Add(ControllerPreset.GetDefault(this));
            UpdateChoices();
            return ControllerPresetsList.Count - 1;
        }
        public void DeletePreset(int index)
        {
            ControllerPresetsList.RemoveAt(index);
            UpdateChoices();
        }
        public void ImportPresets()
        {
            
            SuperController.singleton.GetMediaPathDialog(HandleImportPresets,
                "",SAVEDIR);
        }
        private void HandleImportPresets(string selectedPath)
        {
            var fileContents = SuperController.singleton.ReadFileIntoString(selectedPath);
            JSONNode jsonObject = JSONNode.Parse(fileContents);
            RestoreFromJSON((JSONClass)jsonObject);
        }
        public void ExportPresets()
        {
            FileManagerSecure.CreateDirectory(SAVEDIR);
            var fileBrowserUI = SuperController.singleton.fileBrowserUI;
            fileBrowserUI.SetTitle("Save Controller preset");
            fileBrowserUI.fileRemovePrefix = null;
            fileBrowserUI.hideExtension = false;
            fileBrowserUI.keepOpen = false;
            fileBrowserUI.fileFormat = "json";
            fileBrowserUI.defaultPath = SAVEDIR;
            fileBrowserUI.showDirs = true;
            fileBrowserUI.shortCuts = null;
            fileBrowserUI.browseVarFilesAsDirectories = false;
            fileBrowserUI.Show(HandleExportPresets);
            fileBrowserUI.SetTextEntry(true);
            fileBrowserUI.ActivateFileNameField();
        }
        private void HandleExportPresets(string path)
        {
            var json = GetJSON(false, false, false);
            string fileContents = json.ToString();
            if (!path.ToLower().Contains(".json"))
                path += ".json";
            SuperController.singleton.SaveStringIntoFile(path, fileContents );
        }
    }

    public class MainScreen : DynamicScreen
    {
        private ControllerPresets Parent;
        private int SelectedPreset;
        private bool Renaming;
        private int ShowOptions;

        private JSONStorableStringChooser ShortcutChooser;
        public MainScreen(MVRScript parentScript) : base(parentScript)
        {
            this.Parent = (ControllerPresets)parentScript;
            ShortcutChooser = new JSONStorableStringChooser("Shortcut Key", GetShortcutKeyChoices(), "None", "Shortcut Key", ShortcutChooserChanged);
            SelectedPreset = -1;
            ShowOptions = 0;
        }
        public void DrawScreen()
        {
            if(SelectedPreset == -1)
            {
                var createButton = GetNewButton("Create new Preset", false, Color.green, Color.black);
                createButton.button.onClick.AddListener(() =>
                {
                    int newPresetIndex = Parent.CreateNewPreset();
                    SelectedPreset = newPresetIndex;
                    Renaming = true;
                    RefreshScreen();
                });
                var currentPresets = Parent.GetPresetNameList();
                for (int count = 0; count < currentPresets.Count; count++)
                {
                    var presetButton = GetNewButton(currentPresets[count], true);
                    int index = count;
                    presetButton.button.onClick.AddListener(() => 
                    {
                        ShowOptions = 0;
                        SelectedPreset = index;
                        RefreshScreen();
                    });
                }
                GetNewToggle(new JSONStorableBool("Listen For Shortcut Keys", Parent.ListenForShortcutKeys, (bool newValue) => 
                {
                    Parent.ListenForShortcutKeys = newValue;
                    RefreshScreen();
                }),false);
                var importButton = GetNewButton("Import Presets", false);
                importButton.button.onClick.AddListener(Parent.ImportPresets);
                if(Parent.GetPresetNameList().Count > 0)
                {
                    var exportButton = GetNewButton("Export Presets", false);
                    exportButton.button.onClick.AddListener(Parent.ExportPresets);
                }
                GetNewSpacer(false);
                GetNewTextField(new JSONStorableString("Help", "Created presets are listed as buttons on the right side, " +
                    "click one to edit that preset.\nClick Create new Preset to generate a new preset from the current controller states"),false);
                return;
            }
            var selectedPreset = Parent.GetPreset(SelectedPreset);
            if (Renaming)
            {
                GetNewInputTextField(new JSONStorableString("Preset Name", Parent.GetPreset(SelectedPreset).Name, (string newValue) =>
                {
                    selectedPreset.Name = newValue;
                    Renaming = false;
                    Parent.UpdateChoices();
                    RefreshScreen();
                }), false);
                GetNewButton("Submit", false);
                return;
            }
            var backButton = GetNewButton("Back to Presets List", false, Color.red,Color.white);
            backButton.button.onClick.AddListener(() => 
            {
                SelectedPreset = -1;
                RefreshScreen();
            });
            GetNewButton("Current Preset = " + Parent.GetPreset(SelectedPreset).Name, true);
            var deleteButton = GetNewButton("Delete this Preset", false,Color.black,Color.white);
            deleteButton.button.onClick.AddListener(() =>
            {
                Parent.DeletePreset(SelectedPreset);
                SelectedPreset = -1;
                RefreshScreen();
            });
            UIDynamicButton showButton;
            if (ShowOptions == 0)
                showButton = GetNewButton("Show Interactable On Play", false);
            else if (ShowOptions == 1)
                showButton = GetNewButton("Show Armed For Record", false);
            else
                showButton = GetNewButton("Show States", false);

            showButton.button.onClick.AddListener(() => 
            {
                ShowOptions++;
                if (ShowOptions > 2)
                    ShowOptions = 0;
                RefreshScreen();
            });
            var activateButton = GetNewButton("Activate this Preset", true);
            activateButton.button.onClick.AddListener(() => 
            {
                selectedPreset.Activate();
            });
            if (selectedPreset.ShortcutKey == 0)
                ShortcutChooser.valNoCallback = "None";
            else
                ShortcutChooser.valNoCallback = "Ctrl+"+selectedPreset.ShortcutKey.ToString();
            GetNewFilterablePopup(ShortcutChooser, true);
            GetNewSpacer(false, .85f);
            
            var controllerList = selectedPreset.ControllerStates.Keys.ToList();
            for (int count = 0; count < selectedPreset.ControllerStates.Count; count++)
            {
                var conName = controllerList[count];
                if (ShowOptions == 0)
                {
                    GetNewButton(conName + " (Position)", false, Color.white).button.enabled = false;
                    GetNewButton(conName + " (Rotation)", false).button.enabled = false;
                    ControllerState state = selectedPreset.ControllerStates[conName];
                    PositionState posState = (PositionState)state.PositionState;
                    var posStateButton = GetNewButton(posState.ToString(), true, GetStateColor(posState.ToString()));
                    posStateButton.button.onClick.AddListener(() =>
                    {
                        state.PositionState++;
                        if (state.PositionState >= 8)
                            state.PositionState = 0;
                        RefreshScreen();
                    });
                    RotationState rotState = (RotationState)state.RotationState;
                    var rotStateButton = GetNewButton(rotState.ToString(), true, GetStateColor(rotState.ToString()));
                    rotStateButton.button.onClick.AddListener(() =>
                    {
                        state.RotationState++;
                        if (state.RotationState >= 9)
                            state.RotationState = 0;
                        RefreshScreen();
                    });
                }
                else if (ShowOptions == 1)
                {
                    GetNewButton(conName + "(Interact On Play)", false);
                    bool intOnPlay = selectedPreset.ControllersInteractable[conName];
                    var valueButton = GetNewButton(intOnPlay.ToString(), true);
                    if(intOnPlay)
                    {
                        valueButton.buttonColor = Color.green;
                        valueButton.textColor = Color.white;
                    }
                    else
                    {
                        valueButton.buttonColor = Color.red;
                        valueButton.textColor = Color.white;
                    }
                    valueButton.button.onClick.AddListener(() => 
                    {
                        selectedPreset.ControllersInteractable[conName] = !selectedPreset.ControllersInteractable[conName];
                        RefreshScreen();
                    });
                }
                else
                {
                    GetNewButton(conName + "(Armed for Record)", false);
                    bool armed = selectedPreset.ControllersArmedForRecord[conName];
                    var valueButton = GetNewButton(armed.ToString(), true);
                    if (armed)
                    {
                        valueButton.buttonColor = Color.green;
                        valueButton.textColor = Color.white;
                    }
                    else
                    {
                        valueButton.buttonColor = Color.red;
                        valueButton.textColor = Color.white;
                    }
                    valueButton.button.onClick.AddListener(() =>
                    {
                        selectedPreset.ControllersArmedForRecord[conName] = !selectedPreset.ControllersArmedForRecord[conName];
                        RefreshScreen();
                    });
                }
            }
        }
        public void RefreshScreen()
        {
            CleanUp();
            DrawScreen();
        }
        private Color GetStateColor(string state)
        {
            if (state.Equals("Off"))
                return Color.black;
            else if (state.Equals("On"))
                return Color.white;
            else if (state.Equals("Comply"))
                return Color.green;
            else if (state.Equals("Hold"))
                return Color.Lerp(Color.yellow, Color.red, .5f);
            else if (state.Equals("Lock"))
                return Color.red;
            else if (state.Equals("ParentLink"))
                return Color.blue;
            else if (state.Equals("PhysicsLink"))
                return Color.yellow;
            else
                return Color.gray;
        }
        private List<string> GetShortcutKeyChoices()
        {
            List<string> returnList = new List<string>();
            for (int count = 1; count < 10; count++)
                returnList.Add("Ctrl+" + count.ToString());
            return returnList;
        }
        private void ShortcutChooserChanged(string newValue)
        {
            var preset = Parent.GetPreset(SelectedPreset);
            int value = int.Parse(newValue.Substring(5));
            preset.ShortcutKey = value;
        }
        public void ManualUpdate()
        {

        }
    }

    public class ControllerPreset
    {
        public string Name;
        public int ShortcutKey;
        public Dictionary<string, ControllerState> ControllerStates;
        public Dictionary<string, bool> ControllersInteractable,ControllersArmedForRecord;
        private ControllerPresets Parent;

        public void Activate()
        {
            var controllerList = ControllerStates.Keys.ToList();
            for (int count = 0; count < controllerList.Count; count++)
            {
                var key = controllerList[count];
                FreeControllerV3 controller = (FreeControllerV3)Parent.containingAtom.GetStorableByID(key);
                controller.currentPositionState = (FreeControllerV3.PositionState)ControllerStates[key].PositionState;
                controller.currentRotationState = (FreeControllerV3.RotationState)ControllerStates[key].RotationState;
            }
            controllerList = ControllersInteractable.Keys.ToList();
            for (int count = 0; count < controllerList.Count; count++)
            {
                var key = controllerList[count];
                FreeControllerV3 controller = (FreeControllerV3)Parent.containingAtom.GetStorableByID(key);
                controller.interactableInPlayMode = ControllersInteractable[key];
            }
            controllerList = ControllersArmedForRecord.Keys.ToList();
            for (int count = 0; count < controllerList.Count; count++)
            {
                var key = controllerList[count];
                var controllerAnimation = Parent.containingAtom.GetStorableByID(key.Replace("Control", "Animation"));
                controllerAnimation.SetBoolParamValue("armedForRecord", ControllersArmedForRecord[key]);
            }
            
        }
        public JSONClass GetJSON()
        {
            JSONClass jsc = new JSONClass();
            jsc["Name"] = Name;
            jsc["ShortcutKey"] = ShortcutKey.ToString();

            JSONClass controllerStates = new JSONClass();
            var controllerList = ControllerStates.Keys.ToList();
            for (int count = 0; count < controllerList.Count; count++)
            {
                var key = controllerList[count];
                var state = ControllerStates[key];
                controllerStates[key] = state.GetJSON();
            }
            jsc["ControllerStates"] = controllerStates;
            jsc["ControllersInteractable"] = GetBoolJSONValues(ControllersInteractable);
            jsc["ControllersArmedForRecord"] = GetBoolJSONValues(ControllersArmedForRecord);
            return jsc;
        }
        private JSONClass GetBoolJSONValues(Dictionary<string,bool> data)
        {
            JSONClass returnJSON = new JSONClass();
            var controllerList = data.Keys.ToList();
            for (int count = 0; count < controllerList.Count; count++)
            {
                var key = controllerList[count];
                returnJSON[key] = data[key].ToString();
            }
            return returnJSON;
        }
        public static ControllerPreset GetDefault(ControllerPresets parent)
        {
            var newPreset = new ControllerPreset
            {
                ControllerStates = new Dictionary<string, ControllerState>(),
                ControllersInteractable = new Dictionary<string, bool>(),
                ControllersArmedForRecord = new Dictionary<string, bool>(),
                Name = "New Preset Name",
                ShortcutKey = 0,
                Parent = parent
            };
            for (int count = 0; count < parent.containingAtom.freeControllers.Length; count++)
            {

                string conName = parent.containingAtom.freeControllers[count].name;
                if (conName.ToLower().Contains("hair"))
                    continue;
                else if (conName.Equals("control"))
                    continue;
                else if (conName.Contains("Nipple"))
                    continue;
                else if (conName.Contains("eyeTarget"))
                    continue;
                if (!parent.AtomIsMale && (conName.Contains("penis") || conName.Contains("testes")))
                    continue;
                FreeControllerV3 controller = (FreeControllerV3)parent.containingAtom.GetStorableByID(conName);
                newPreset.ControllerStates.Add(conName, new ControllerState
                {
                    PositionState = (int)controller.currentPositionState,
                    RotationState = (int)controller.currentRotationState
                });
                newPreset.ControllersInteractable.Add(conName, controller.interactableInPlayMode);
                var controllerAnimation = parent.containingAtom.GetStorableByID(conName.Replace("Control", "Animation"));
                newPreset.ControllersArmedForRecord.Add(conName, controllerAnimation.GetBoolJSONParam("armedForRecord").valNoCallback);
            }
            return newPreset;
        }
    }
    public class ControllerState
    {
        public int PositionState;
        public int RotationState;
        public JSONClass GetJSON()
        {
            JSONClass jsc = new JSONClass();
            jsc["PositionState"] = PositionState.ToString();
            jsc["RotationState"] = RotationState.ToString();
            return jsc;
        }
    }

    public abstract class DynamicScreen
    {
        protected List<UIDynamicSlider> sliders = new List<UIDynamicSlider>();
        protected List<UIDynamicToggle> toggles = new List<UIDynamicToggle>();
        protected List<UIDynamicTextField> textFields = new List<UIDynamicTextField>();
        protected List<UIDynamicPopup> popups = new List<UIDynamicPopup>();
        protected List<UIDynamicButton> buttons = new List<UIDynamicButton>();
        protected List<UIDynamic> spacers = new List<UIDynamic>();

        protected MVRScript ParentScript;

        public DynamicScreen(MVRScript parentScript)
        {
            this.ParentScript = parentScript;
        }

        public void CleanUp()
        {
            for (int i = 0; i < sliders.Count; i++)
            {
                UIDynamicSlider slider = sliders[i];
                if (slider != null)
                    ParentScript.RemoveSlider(slider);
            }
            for (int i = 0; i < textFields.Count; i++)
            {
                UIDynamicTextField textField = textFields[i];
                if (textField != null)
                    ParentScript.RemoveTextField(textField);
            }
            for (int i = 0; i < toggles.Count; i++)
            {
                UIDynamicToggle toggle = toggles[i];
                if (toggle != null)
                    ParentScript.RemoveToggle(toggle);
            }
            for (int i = 0; i < popups.Count; i++)
            {
                UIDynamicPopup popup = popups[i];
                if (popup != null)
                    ParentScript.RemovePopup(popup);
            }
            for (int i = 0; i < buttons.Count; i++)
            {
                UIDynamicButton button = buttons[i];
                if (button != null)
                    ParentScript.RemoveButton(button);
            }
            for (int count = 0; count < spacers.Count; count++)
            {
                var spacer = spacers[count];
                if (spacer != null)
                    ParentScript.RemoveSpacer(spacer);
            }
            sliders.Clear();
            textFields.Clear();
            toggles.Clear();
            popups.Clear();
            buttons.Clear();
            spacers.Clear();
        }
        public UIDynamic GetNewSpacer(bool rightSide, float size = 3)
        {
            var spacer = ParentScript.CreateSpacer(rightSide);
            spacer.height = 65 * size;
            spacers.Add(spacer);
            return spacer;
        }
        public UIDynamicSlider GetNewSlider(JSONStorableFloat jsf, bool rightSide)
        {
            var slider = ParentScript.CreateSlider(jsf, rightSide);
            sliders.Add(slider);
            return slider;
        }
        public UIDynamicToggle GetNewToggle(JSONStorableBool jsb, bool rightSide)
        {
            var toggle = ParentScript.CreateToggle(jsb, rightSide);
            toggles.Add(toggle);
            return toggle;
        }
        public UIDynamicTextField GetNewTextField(JSONStorableString jss, bool rightSide,int height = 0)
        {
            var textField = ParentScript.CreateTextField(jss, rightSide);
            textFields.Add(textField);
            if(height != 0)
                textField.height = height;
            return textField;
        }
        public UIDynamicPopup GetNewScrollablePopup(JSONStorableStringChooser jsc, bool rightSide)
        {
            var popup = ParentScript.CreateScrollablePopup(jsc, rightSide);
            popups.Add(popup);
            return popup;
        }
        public UIDynamicPopup GetNewFilterablePopup(JSONStorableStringChooser jsc, bool rightSide)
        {
            var popup = ParentScript.CreateFilterablePopup(jsc, rightSide);
            popups.Add(popup);
            return popup;
        }
        public UIDynamicButton GetNewButton(string label, bool rightSide)
        {
            var button = ParentScript.CreateButton(label, rightSide);
            buttons.Add(button);
            return button;
        }
        public UIDynamicButton GetNewButton(string label, bool rightSide, Color buttonColor)
        {
            var button = GetNewButton(label, rightSide);
            button.buttonColor = buttonColor;
            if (buttonColor == Color.blue || buttonColor == Color.black || buttonColor == Color.red || buttonColor == Color.green)
                button.textColor = Color.white;
            return button;
        }
        public UIDynamicButton GetNewButton(string label, bool rightSide, Color buttonColor, Color textColor)
        {
            var button = GetNewButton(label, rightSide, buttonColor);
            button.textColor = textColor;
            return button;
        }
        public UIDynamicTextField GetNewInputTextField(JSONStorableString jss, bool rightSide, bool disableInput = false)
        {
            var textfield = GetNewTextField(jss, rightSide);
            var input = textfield.gameObject.AddComponent<InputField>();
            var rect = input.GetComponent<RectTransform>().sizeDelta = new Vector2(1f, 0.4f);
            input.textComponent = textfield.UItext;
            if (disableInput)
                input.DeactivateInputField();
            jss.inputField = input;
            return textfield;
        }
    }
}
